import { Component, OnInit, ChangeDetectorRef, Renderer2, ElementRef, ViewChild, inject, QueryList, ViewChildren , Input } from '@angular/core';
import {MatOption, provideNativeDateAdapter} from "@angular/material/core";
import {MatIconModule} from '@angular/material/icon';
import {FormBuilder, FormControl, FormGroup, FormsModule, ReactiveFormsModule, Validators} from '@angular/forms';
import {CdkTextareaAutosize} from "@angular/cdk/text-field";
import {MatAutocomplete, MatAutocompleteSelectedEvent, MatAutocompleteTrigger} from "@angular/material/autocomplete";
import {MatCard, MatCardContent, MatCardSubtitle , MatCardFooter, MatCardHeader, MatCardTitle , MatCardModule} from "@angular/material/card";
import {MatButton, MatFabButton, MatIconButton} from "@angular/material/button";
import {MatFormField, MatHint, MatLabel, MatSuffix,MatFormFieldModule} from "@angular/material/form-field";
import {MatIcon} from "@angular/material/icon";
import {MatInput} from "@angular/material/input";
import {MatTooltip} from "@angular/material/tooltip";
import {DatePipe, NgForOf, NgIf, NgStyle} from "@angular/common";
import {
  MatCell,
  MatCellDef,
  MatColumnDef,
  MatHeaderCell, MatHeaderCellDef,
  MatHeaderRow,
  MatHeaderRowDef,
  MatRow, MatRowDef, MatTable
} from "@angular/material/table";
import {MatDatepicker, MatDatepickerInput, MatDatepickerToggle} from "@angular/material/datepicker";
import {MatSelect} from "@angular/material/select";
import {SoloNumerosDirective} from "../../service/directivas_service/soloNumeros/solo-numeros.directive";
import {MatSnackBar, MatSnackBarHorizontalPosition, MatSnackBarVerticalPosition} from "@angular/material/snack-bar";
import {ConformidadHelper} from "../conformidad/helper/conformidad.helper";
import {MatDialog} from "@angular/material/dialog";
import { ConfirmarEliminacionComponent  } from '../conformidad/modalEliminarPasaje/eliminarPasaje.componente';
import { modalArticuloOC } from '../conformidad/modalArticuloOC/modalArticuloOC.componente'
import {modalRequerimientoComponent} from "../conformidad/modalRequerimiento/modalRequerimiento.componente";

interface interArticulo{
  index: number;
  codigoArticulo : string;
  nombre: string;
  unidad: string;
  cantidad: number;
  estado : number; // 1 = BUENO ESTADO , 2 = OBSERVACIÓN , 3 = MAL ESTADO
  isNew: boolean;
  isEdit : boolean;
  acciones: boolean;
}

interface listaArticulos{
  index: number,
  codigo : string,
  nombre: string,
  unidad: string,
  cantidad: number,
  estado: number,
}

interface listadoComun {
  codigo: string;
  nombre: string;
}

@Component({
  selector: 'ordensalida-component',
  templateUrl: './ordensalida.componente.html',
  styleUrls: ['./ordensalida.componente.css'],
  providers: [provideNativeDateAdapter()],
  standalone: true,
  imports: [
    CdkTextareaAutosize,
    FormsModule,
    MatAutocomplete,
    MatAutocompleteTrigger,
    MatCard,
    MatCardContent,
    MatCardSubtitle,
    MatFabButton,
    MatFormField,
    MatIcon,
    MatInput,
    MatLabel,
    MatOption,
    MatTooltip,
    NgForOf,
    ReactiveFormsModule,
    MatButton,
    MatCell,
    MatCellDef,
    MatColumnDef,
    MatDatepicker,
    MatDatepickerInput,
    MatDatepickerToggle,
    MatHeaderCell,
    MatHeaderRow,
    MatHeaderRowDef,
    MatHint,
    MatIconButton,
    MatRow,
    MatRowDef,
    MatSelect,
    MatSuffix,
    MatTable,
    NgIf,
    SoloNumerosDirective,
    NgStyle,
    MatHeaderCellDef,
    MatIconModule,
    MatFormFieldModule,
    MatCardModule,
    MatCardHeader,
    MatCardTitle,
    MatCardFooter,
  ]
})

export class ordenSalidaComponent implements OnInit {

  @Input() datosDesdeConformidad: any; // Variable para recibir datos

  private barraAlerta = inject(MatSnackBar);
  alertaPosicionHorizontal: MatSnackBarHorizontalPosition = 'center';
  alertaPosicionVertical: MatSnackBarVerticalPosition = 'bottom';
  alertaDuracion: number = 3;


  //FORM GROUP
  public datos_materialSobrante: FormGroup = new FormGroup<any>({
    descMaterial : new FormControl<String | null>(null,[Validators.maxLength(100)]),
  });
  //FORM GROUP

  // INICIALIZACION DATOS
  activarNuevoArticulo: boolean = true;
  indexArticulo: number = 0;
  public dataSource: listaArticulos[] = [];
  listUnidades : listadoComun[] = [];
  listEstadoArticulo: listadoComun[] = [];
  columnasArticulos: string[] = ['nombre', 'unidad', 'cantidad', 'estado', 'accion'];
  articulosList: any[] = [];
  // INICIALIZACION DATOS

  // OBTENER DATOS DEL FORM GROUP
  formGroupMS(accion: number , formControlName : string , formControl : FormControl = new FormControl()): any {

    if(accion === 1){
      return this.datos_materialSobrante.addControl(formControlName, formControl);
    }else if(accion === 2){
      //Obtener un formControl del formGroup y validar si existe caso contrario retornar null
      return this.datos_materialSobrante.get(formControlName);
    }else if(accion === 3){
      return this.datos_materialSobrante.removeControl(formControlName);
    }
  }

  constructor(
    private conformidadHelper: ConformidadHelper,
    private renderer: Renderer2,
    private detectorChange : ChangeDetectorRef,
    public dialog: MatDialog,
    private datePipe: DatePipe,
  ) {
    this.articulosList = [];
  }

  async ngOnInit(): Promise<void> {
    this.dataSource = [];
    this.listUnidades = this.datosDesdeConformidad.map((item: any) => ({codigo: item.CODIGO , nombre: item.NOMBRE}));
    this.listEstadoArticulo = [{ codigo: '1', nombre: 'VERDE'}, { codigo: '2', nombre: 'AMARILLO' },{ codigo: '3', nombre: 'ROJO' } ];
  }

  onEventoAgregarArticulo(event : any){
    const dialogRef = this.dialog.open(modalArticuloOC,{
      //width: '100%',
      //height: '80%',
      data: { Unidades : this.listUnidades , Estados : this.listEstadoArticulo , Editar: false }
    });

    dialogRef.afterClosed().subscribe(result => {
      console.log(result);

      if(result){
        const NuevaFila : listaArticulos = {
          index : this.indexArticulo++,
          codigo: result.codigoArticulo,
          nombre: result.nombreArticulo,
          unidad: result.unidadArticulo,
          cantidad: result.cantidadArticulo,
          estado: result.estadoArticulo
        };

        this.dataSource = [...this.dataSource,NuevaFila];
      }

    });
  }

  onEventoEditarArticulo(event: any, elemento : any) {

    const DatoArticulo = this.dataSource.find((item) => item.index === elemento.index);

    const dialogRef = this.dialog.open(modalArticuloOC, {
      //width: '100%',
      //height: '80%',
      data: {Unidades: this.listUnidades, Estados: this.listEstadoArticulo , DatoArticulo , Editar: true}
    });

    dialogRef.afterClosed().subscribe(result => {
      console.log(result);

      if(result){
        const NuevaFila : listaArticulos = {
          index : elemento.index,
          codigo: result.codigoArticulo,
          nombre: result.nombreArticulo,
          unidad: result.unidadArticulo,
          cantidad: result.cantidadArticulo,
          estado: result.estadoArticulo
        };

        this.dataSource[elemento.index] = {
          index: NuevaFila.index,
          codigo : NuevaFila.codigo,
          nombre: NuevaFila.nombre,
          unidad: NuevaFila.unidad,
          cantidad: NuevaFila.cantidad,
          estado: NuevaFila.estado
        };

        this.dataSource = [...this.dataSource];
      }

    });
  }

  onEventoEliminarArticulo(event: any, elemento : any) {
    const dialogRef = this.dialog.open(ConfirmarEliminacionComponent);

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        const index = elemento.index;
        if (index >= 0) {
          this.dataSource = this.dataSource.filter((item) => item.index !== elemento.index);
        }
      }
    });
  }

  async onBusquedaArticulo(event: any , index : number) { // OMITIR LA BUSQUEDA POR AHORA YA QUE NO CONOCEN LOS ARTICULOS

    const busquedaArt = event.target.value;

    const jsonParametrosArti = {
      accion : 4,
      nombreValor : busquedaArt,
    };

    const articulos_list = await this.conformidadHelper.listadoGeneralHorizon(jsonParametrosArti) ;
    const mapeoResultado = articulos_list.map((item: any) => ({
      codigo: item.codigoProducto,
      nombre : item.nombreProducto,
      unidad: item.unidad
    }));

    this.articulosList = mapeoResultado;
  }

  onSeleccionArticulo(event: MatAutocompleteSelectedEvent,index : number) { // OMITIR LA BUSQUEDA POR AHORA YA QUE NO CONOCEN LOS ARTICULOS

    //AL SELECCIONAR , SE OPTIENE EL VALUE
    const selectedOption = event.option.value;

    //SE BUSCA EL VALUE DENTRO DEL LISTADO DE ARTICULOS
    const articulo = this.articulosList.find((item) => item.codigo === selectedOption);

    //console.log(articulo);
    //SE OPTIENE EL NOMBRE PARA PONERLO EN EL INPUT
    this.formGroupMS(2,'articulo'+index)?.setValue(articulo.nombre);
    this.formGroupMS(2,'codigoArticulo'+index)?.setValue(articulo.codigo);
    this.formGroupMS(2,'unidad'+index)?.setValue(articulo.unidad);
   // this.codigoUnidad = articulo.unidad;

    //SE VUELVE A LIMPIAR EL LISTADO DE ARTICULOS PARA DEJAR PASO A OTRA FUTURA BUSQUEDA.
    this.articulosList = [];

  }

  formatoEstado(estado: any) {
    return this.listEstadoArticulo.find((item) => item.codigo === estado)?.nombre;
  }
}
