import {Injectable} from "@angular/core";
import axios from "axios";
import {environment} from "../../../environment/env";

@Injectable({
  providedIn: 'root'
})
export class ConformidadService {

  t_asistencia_rest_link = environment.t_asistencia_rest_link;
  t_horizon_rest_link = environment.horizon_services_link;
  t_facturacion_electronica_link = environment.facturacion_electronica_link;
  value_idDrive = environment.ID_CARPETA_DRIVE_LOGISTICA;
  value_correoDrive = environment.CORREO_DRIVE_LOGISTICA;
  t_redireccionamiento = environment.redireccionamiento;

  constructor() {
  }

  async listadoGeneral(opcion: number) {
    try {
      const respuesta = await axios.post(this.t_asistencia_rest_link + '/api/v1/Conformidad/listadoGeneral',
        {
          v_opcion: opcion,
        });
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async listadoGeneralHorizon(json: any) {

    const parametros =[
      json.accion || 0,
      json.nombreValor || '',
      json.sede || '',
    ]

    try {
      const respuesta = await axios.post(this.t_horizon_rest_link + '/api/procedure/Procedimiento',
        {
          database: "sqlserver",
          procedure :"USP_GENERAL_FORMULARIO",
          params : parametros
        },
        {
          headers: {
            'Content-Type': 'application/json'
          }
        }
      );
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async gestionConformidad(json: any) {

    const Parametros = {
      opcion : json.opcion || 0,
      codconformidad : json.codconformidad || 0,
      codper : json.codper || 0,
      codlocal : json.codlocal || '',
      fechainicio : json.fechainicio || '1999-01-01',
      fechafin : json.fechafin || '1999-01-01',
      descproblema : json.descproblema || '',
      desccausa : json.desccausa || '',
      codtipotrabajo: json.codtipotrabajo || 0 ,
      desctrabajo: json.desctrabajo || '',
      pasaje : json.pasaje || 0,
      codrequerimiento : json.codrequerimiento || 0,
      codordensalida : json.codordensalida || 0,
      minhoratrabajo : json.minhoratrabajo || '',
      maxhoratrabajo : json.maxhoratrabajo || '',
    };

    try {
      const respuesta = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/registrarConformidad', Parametros);
      //console.log(respuesta);
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async creacionCarpeta(json: any) {
    let respuesta: any;
    const url = `${this.t_redireccionamiento}/google/drive/carpeta`;

    const jsonData =  {
      carpeta_nombre : 'CONFORMIDAD_' + json.codconformidad ,
      carpeta_id : this.value_idDrive ,
      correo : this.value_correoDrive
    }

    const controller = new AbortController();
    const timeout = setTimeout(() => controller.abort(), 30000);  // 30 segundos
    try {
      const response = await fetch(url, {
        method: 'POST',
        body: JSON.stringify(jsonData),
        signal: controller.signal,  // 30 segundos de timeout
      });

      const statusCode = response.status;
      const responseBody = await response.json();

      clearTimeout(timeout);  // Limpiar, REVISA SI ESTO ESTA BIEN O QUITARLO

      if (statusCode === 200) {
        respuesta = {status: true, message: "Carpeta creada" , idCarpeta : responseBody.data.carpeta_id , nombreCarpeta : responseBody.data.carpeta_nombre};
      }else{
        respuesta = {status: false, message: "Carpeta no creada"};
      }
    }catch (error) {
      console.error('Error al registrar el archivo:', error);
      //throw error;
      respuesta = {status: false, message: error};
    }

    return respuesta;
  }

  async subidaEvidencia2(json:any){
    const url = `${this.t_redireccionamiento}/google/drive`;
    let respuesta: any;

    for(let i = 0 ; i < json.length ; i++){

      const datos = json[i];

      const formData = new FormData();
      formData.append('archivo_nombre', datos.nombreArchivo);
      formData.append('archivo_binario', datos.archivo);
      formData.append('archivo_id_actual', '');
      formData.append('carpeta_id', datos.idCarpeta || this.value_idDrive);
      formData.append('correo', this.value_correoDrive);
      formData.append('app_nombre', 'ORDEN_COMPRA'); // USANDO ESTE NOMBRE DE APP(ORDEN_COMPRA) YA QUE NO CUENTA CON LA CONFIGURACION DE SUBCARPETAS
      formData.append('carpeta_nombre', datos.tipoEvidencia === 1 ? 'EVIDENCIAS_PROBLEMAS' :
        datos.tipoEvidencia === 2 ? 'EVIDENCIAS_SOLUCIONES' : 'EVIDENCIAS_EXTRAS' );
      //const controller = new AbortController();
      //const timeout = setTimeout(() => controller.abort(), 60000);  // 30 segundos
      try {
        const response = await fetch(url, {
          method: 'POST',
          body: formData,
          //signal: controller.signal,  // 30 segundos de timeout
        });

        const statusCode = response.status;
        const responseBody = await response.json();
        console.log(response)
        //clearTimeout(timeout);  // Limpiar, REVISA SI ESTO ESTA BIEN O QUITARLO

        if (statusCode === 200) {

          const registroEvidencia = await this.registroDocumento({
            codconformidad: datos.codconformidad || 0,
            tipoEvidencia: datos.tipoEvidencia || 0,
            idDrive: responseBody.data.archivo_id || '',
            enlaceDrive: responseBody.data.archivo_url_vista || '',
            enlaceDescarga: responseBody.data.archivo_url || '',
            nombreDocumento: datos.nombreArchivo || ''
          });

          if (responseBody?.status) {
            respuesta = {status: true, message: "Imagenes subidas"};
          } else {
            respuesta = {status: false, message: "Fallo en subir imagene"};
            break;
          }

        } else {
          respuesta = {status: false, message: "Fallo en subir imagene " + response};
          break;
        }

        //return respuesta;
      } catch (error) {
        console.error('Error al registrar el archivo:', error);
        //throw error;
        respuesta = {status: false, message: error};
      }

    }

    return respuesta;

  }

  async registroDocumento(json: any) {
    const parametros = {
      accion : 1,
      codconformidad: json.codconformidad || 0,
      tipoEvidencia: json.tipoEvidencia || 0,
      idDrive: json.idDrive || '',
      enlaceDrive: json.enlaceDrive || '',
      enlaceDescarga: json.enlaceDescarga || '',
      nombreDocumento: json.nombreDocumento || ''
    };

    try {
      const respuesta = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/registroDocumentos', parametros);

      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async gestionOrdenSalida(json : any){

    const parametros = {
      accion : json.accion || 0,
      codOrdenSalida : json.codOrdenSalida || 0,
      codlocal : json.codlocal || '',
      descripcion: json.descripcion || '',
      fechaRegistro : json.fechaRegistro || '',
      codArticulo : json.codArticulo || '',
      nombreArticulo : json.nombreArticulo || '',
      unidad : json.unidad || '-',
      cantidad : json.cantidad || 0,
      estado : json.estado || 0,

    };

    try {
      const respuesta = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/gestionOrdenSalida', parametros);
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

}
